@@ -1,5 +1,5 @@
 //
-// $Id: SpectrumListFactory.cpp 6478 2014-07-08 20:01:38Z chambm $
+// $Id: SpectrumListFactory.cpp 9095 2015-11-01 01:14:24Z pcbrefugee $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -35,6 +35,8 @@
 #include "pwiz/analysis/spectrum_processing/SpectrumList_PrecursorRecalculator.hpp"
 #include "pwiz/analysis/spectrum_processing/SpectrumList_PrecursorRefine.hpp"
 #include "pwiz/analysis/spectrum_processing/SpectrumList_MZWindow.hpp"
+#include "pwiz/analysis/spectrum_processing/SpectrumList_MZRefiner.hpp"
+#include "pwiz/analysis/spectrum_processing/SpectrumList_LockmassRefiner.hpp"
 #include "pwiz/analysis/spectrum_processing/SpectrumList_MetadataFixer.hpp"
 #include "pwiz/analysis/spectrum_processing/SpectrumList_TitleMaker.hpp"
 #include "pwiz/analysis/spectrum_processing/PrecursorMassFilter.hpp"
@@ -42,6 +44,7 @@
 #include "pwiz/analysis/spectrum_processing/SpectrumList_ZeroSamplesFilter.hpp"
 #include "pwiz/analysis/spectrum_processing/MS2NoiseFilter.hpp"
 #include "pwiz/analysis/spectrum_processing/MS2Deisotoper.hpp"
+#include "pwiz/utility/misc/Filesystem.hpp"
 #include "pwiz/utility/misc/Std.hpp"
 
 
@@ -57,16 +60,72 @@ using namespace pwiz::util;
 namespace {
 
 
+inline const char* cppTypeToNaturalLanguage(short T) { return "a small integer"; }
+inline const char* cppTypeToNaturalLanguage(int T) { return "an integer"; }
+inline const char* cppTypeToNaturalLanguage(long T) { return "an integer"; }
+inline const char* cppTypeToNaturalLanguage(unsigned short T) { return "a small positive integer"; }
+inline const char* cppTypeToNaturalLanguage(unsigned int T) { return "a positive integer"; }
+inline const char* cppTypeToNaturalLanguage(unsigned long T) { return "a positive integer"; }
+inline const char* cppTypeToNaturalLanguage(float T) { return "a real number"; }
+inline const char* cppTypeToNaturalLanguage(double T) { return "a real number"; }
+inline const char* cppTypeToNaturalLanguage(bool T) { return "true or false"; }
+
+
+/// parses a lexical-castable key=value pair from a string of arguments which may also include non-key-value strings;
+/// if the key is not in the argument string, defaultValue is returned;
+/// if bad_lexical_cast is thrown, it is converted into a sensible error message
+template <typename ArgT>
+ArgT parseKeyValuePair(string& args, const string& tokenName, const ArgT& defaultValue)
+{
+    try
+    {
+        size_t keyIndex = args.rfind(tokenName);
+        if (keyIndex != string::npos)
+        {
+            size_t valueIndex = keyIndex + tokenName.length();
+            if (valueIndex < args.length())
+            {
+                string valueStr;
+                try
+                {
+                    valueStr = args.substr(valueIndex, args.find(" ", valueIndex) - valueIndex);
+                    ArgT value = lexical_cast<ArgT>(valueStr);
+                    args.erase(keyIndex, args.find(" ", valueIndex) - keyIndex);
+                    return value;
+                }
+                catch (bad_lexical_cast&)
+                {
+                    throw runtime_error("error parsing \"" + valueStr + "\" as value for \"" + tokenName + "\"; expected " + cppTypeToNaturalLanguage(defaultValue));
+                }
+            }
+        }
+        return defaultValue;
+    }
+    catch (exception& e)
+    {
+        throw runtime_error(string("[parseKeyValuePair] ") + e.what());
+    }
+}
+
+/// parses a lexical-castable key=value pair from a string of arguments which may also include non-key-value strings;
+/// if the key is not in the argument string or if bad_lexical_cast is thrown, a default-constructed ArgT is returned
+template <typename ArgT>
+ArgT parseKeyValuePair(string& args, const string& tokenName)
+{
+    return parseKeyValuePair<ArgT>(args, tokenName, ArgT());
+}
+
+
 //
 // each SpectrumListWrapper has a filterCreator_* function, 
 // and an entry in the jump table below
 //
 
 
-typedef SpectrumListPtr (*FilterCreator)(const MSData& msd, const string& arg);
+typedef SpectrumListPtr (*FilterCreator)(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr);
 typedef const char *UsageInfo[2];  // usage like <int_set>, and details
 
-SpectrumListPtr filterCreator_index(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_index(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     IntegerSet indexSet;
     indexSet.parse(arg);
@@ -80,7 +139,7 @@ UsageInfo usage_index = {"<index_value_set>",
     "  <index_value_set> is an int_set of indexes."
 };
 
-SpectrumListPtr filterCreator_scanNumber(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_scanNumber(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     IntegerSet scanNumberSet;
     scanNumberSet.parse(arg);
@@ -94,7 +153,7 @@ UsageInfo usage_scanNumber = {"<scan_numbers>",
     "<scan_numbers> is an int_set of scan numbers to be kept."
 };
 
-SpectrumListPtr filterCreator_scanEvent(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_scanEvent(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     IntegerSet scanEventSet;
     scanEventSet.parse(arg);
@@ -110,7 +169,7 @@ UsageInfo usage_scanEvent = {"<scan_event_set>","This filter selects spectra by
             "the Thermo Xcalibur glossary as: \"a mass spectrometer scan that is defined by choosing the necessary scan parameter "
             "settings. Multiple scan events can be defined for each segment of time.\"."};
 
-SpectrumListPtr filterCreator_scanTime(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_scanTime(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     double scanTimeLow = 0;
     double scanTimeHigh = 0;
@@ -134,14 +193,14 @@ UsageInfo usage_scanTime = {"<scan_time_range>",
     "  <scan_time_range> is a time range, specified in seconds.  For example, to select only spectra within the "
     "second minute of the run, use \"scanTime [60-119.99]\"."};
 
-SpectrumListPtr filterCreator_sortScanTime(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_sortScanTime(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     return SpectrumListPtr(new SpectrumList_Sorter(msd.run.spectrumListPtr,
                                                    SpectrumList_SorterPredicate_ScanStartTime()));
 }
 UsageInfo usage_sortScanTime = {"","This filter reorders spectra, sorting them by ascending scan start time."};
 
-SpectrumListPtr filterCreator_scanSummer(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_scanSummer(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
 
     // defaults
@@ -188,7 +247,7 @@ UsageInfo usage_scanSummer = {"[precursorTol=<precursor tolerance>] [scanTimeTol
     "and <scan time tolerance> (default: 10 secs.). Its use is intended for some Waters DDA data, where sub-scans " 
     "should be summed together to increase the SNR. This filter has only been tested for Waters data."};
 
-SpectrumListPtr filterCreator_nativeCentroid(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_nativeCentroid(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
 
     // by default assume we are using the low-quality localMax code unless explicitly told otherwise
@@ -319,7 +378,7 @@ UsageInfo usage_nativeCentroid = {"[<PickerType> [snr=<minimum signal-to-noise r
  *  Handler for --filter zeroSamples removeExtra|addMissing[=FlankingZeroCount] [mslevels]
  *
  **/
-SpectrumListPtr filterCreator_ZeroSamples(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_ZeroSamples(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     istringstream parser(arg);
     string action;
@@ -369,7 +428,7 @@ UsageInfo usage_zeroSamples = {"<mode> [<MS_levels>]",
  *  <true|false> remove neutral loss species from charge reduced precursor
  *  float_val <MZ|PPM> matching tolerance -- floating point value, followed by units (example: 3.1 MZ)
  */
-SpectrumListPtr filterCreator_ETDFilter(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_ETDFilter(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     istringstream parser(arg);
 
@@ -409,13 +468,14 @@ SpectrumListPtr filterCreator_ETDFilter(const MSData& msd, const string& arg)
 }
 UsageInfo usage_ETDFilter = {"[<removePrecursor> [<removeChargeReduced> [<removeNeutralLoss> [<blanketRemoval> [<matchingTolerance> ]]]]]",
     "Filters ETD MSn spectrum data points, removing unreacted precursors, charge-reduced precursors, and neutral losses.\n"
-    "  <removePrecursor> - specify \"true\" to remove unreacted precursor (default is \"false\")\n"
-    "  <removeChargeReduced> - specify \"true\" to remove charge reduced precursor (default is \"false\")\n"
-    "  <removeNeutralLoss> - specify \"true\" to remove neutral loss species from charge reduced precursor (default is \"false\")\n"
-    "  <matchingTolerance> - specify matching tolerance in MZ or PPM (examples: \"3.1 MZ\" (the default) or \"2.2 PPM\")"
+    "  <removePrecursor> - if \"true\", remove unreacted precursor (default is \"true\")\n"
+    "  <removeChargeReduced> - if \"true\", remove charge reduced precursor (default is \"true\")\n"
+    "  <removeNeutralLoss> - if \"true\", remove neutral loss species from charge reduced precursor (default is \"true\")\n"
+    "  <blanketRemoval> - if \"true\", remove neutral losses in a charge-scaled 60 Da swath rather than only around known loss species (default is \"true\")\n"
+    "  <matchingTolerance> - specify matching tolerance in m/z or ppm (examples: \"3.1 mz\" (the default) or \"2.2 ppm\")"
 };
 
-SpectrumListPtr filterCreator_MS2Denoise(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_MS2Denoise(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     istringstream parser(arg);
 
@@ -457,7 +517,7 @@ UsageInfo usage_MS2Denoise = {"[<peaks_in_window> [<window_width_Da> [multicharg
     "Proteomics, 9, 4978-4984, 2009.\n"
 };
 
-SpectrumListPtr filterCreator_MS2Deisotope(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_MS2Deisotope(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     istringstream parser(arg);
 
@@ -577,7 +637,7 @@ struct StripIonTrapSurveyScans : public SpectrumList_Filter::Predicate
     }
 };
 
-SpectrumListPtr filterCreator_stripIT(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_stripIT(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     return SpectrumListPtr(new 
         SpectrumList_Filter(msd.run.spectrumListPtr, StripIonTrapSurveyScans()));
@@ -585,7 +645,7 @@ SpectrumListPtr filterCreator_stripIT(const MSData& msd, const string& arg)
 UsageInfo usage_stripIT={"","This filter rejects ion trap data spectra with MS level 1."};
 
 
-SpectrumListPtr filterCreator_precursorRecalculation(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_precursorRecalculation(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     return SpectrumListPtr(new SpectrumList_PrecursorRecalculator(msd));
 }
@@ -594,7 +654,138 @@ UsageInfo usage_precursorRecalculation = {"","This filter recalculates the precu
     "although it does not use any 3rd party (vendor DLL) code.  Since the time the code was written, Thermo has since fixed "
     "up its own estimation in response, so it's less critical than it used to be (though can still be useful)."};
 
-SpectrumListPtr filterCreator_precursorRefine(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_mzRefine(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
+{
+    // Example string:
+    // "mzRefiner input1.pepXML input2.mzid msLevels=1- thresholdScore=specEValue thresholdValue=1e-10 thresholdStep=10 maxSteps=3"
+
+    istringstream parser(arg);
+    // expand the filenames by globbing to handle wildcards
+    vector<bfs::path> globbedFilenames;
+    string thresholdCV = "MS-GF:SpecEValue"; // Remove this default?
+    string thresholdSet = "-1e-10";          // Remove this default?
+    double thresholdStep = 0.0;
+    int maxSteps = 0;
+    string msLevelSets = "1-";
+
+    string nextStr;
+    while (parser >> nextStr)
+    {
+
+        if (nextStr.rfind('=') == string::npos)
+        {
+            // Add to ident file list, and check for existence?
+            // Remove quotes that may be used to encapsulate a path with spaces
+            if ((bal::starts_with(nextStr, "\'") && bal::ends_with(nextStr, "\'")) || (bal::starts_with(nextStr, "\"") && bal::ends_with(nextStr, "\"")))
+            {
+                // Remove the first and last characters
+                nextStr = nextStr.substr(1, nextStr.length() - 2);
+            }
+            // expand the filenames by globbing to handle wildcards
+            if (expand_pathmask(bfs::path(nextStr), globbedFilenames) == 0)
+                cout << "[mzRefiner] no files found matching \"" << nextStr << "\"" << endl;
+        }
+        else
+        {
+            string keyword = nextStr.substr(0, nextStr.rfind('='));
+            string paramVal = nextStr.substr(nextStr.rfind('=') + 1);
+            if (keyword == "msLevels")
+            {
+                // use an IntegerSet
+                msLevelSets = paramVal;
+            }
+            else if (keyword == "thresholdScore")
+            {
+                thresholdCV = paramVal;
+            }
+            else if (keyword == "thresholdValue")
+            {
+                thresholdSet = paramVal;
+            }
+            else if (keyword == "thresholdStep")
+            {
+                thresholdStep = boost::lexical_cast<double>(paramVal);
+            }
+            else if (keyword == "maxSteps")
+            {
+                maxSteps = boost::lexical_cast<int>(paramVal);
+            }
+        }
+    }
+    // expand the filenames by globbing to handle wildcards
+    vector<string> files;
+    BOOST_FOREACH(const bfs::path& filename, globbedFilenames)
+        files.push_back(filename.string());
+
+    IntegerSet msLevelsToRefine;
+    msLevelsToRefine.parse(msLevelSets);
+
+    string identFilePath = "";
+
+    vector<string> possibleDataFiles;
+    string lastSourceFile;
+
+    BOOST_FOREACH(const SourceFilePtr& sf, msd.fileDescription.sourceFilePtrs)
+    {
+        lastSourceFile = sf->name;
+    }
+
+    lastSourceFile = lastSourceFile.substr(0, lastSourceFile.rfind(".gz")); // remove a ".gz", if there is one
+    lastSourceFile = lastSourceFile.substr(0, lastSourceFile.rfind(".")); // remove the extension
+    possibleDataFiles.push_back(lastSourceFile);
+    possibleDataFiles.push_back(msd.run.id);
+
+    // Search for a file that matches the MSData file name, then search for one matching the dataset if not found.
+    // Load identfiles, and look at mzid.DataCollection.Inputs.SpectraData.name?
+    BOOST_FOREACH(string &dataFile, possibleDataFiles)
+    {
+        BOOST_FOREACH(string &file, files)
+        {
+            if (file.find(dataFile) != string::npos)
+            {
+                identFilePath = file;
+                break;
+            }
+        }
+        if (identFilePath != "")
+        {
+            break;
+        }
+    }
+
+    return SpectrumListPtr(new SpectrumList_MZRefiner(msd, identFilePath, thresholdCV, thresholdSet, msLevelsToRefine, thresholdStep, maxSteps, ilr));
+}
+UsageInfo usage_mzRefine = { "input1.pepXML input2.mzid [msLevels=<1->] [thresholdScore=<CV_Score_Name>] [thresholdValue=<floatset>] [thresholdStep=<float>] [maxSteps=<count>]", "This filter recalculates the m/z and charges, adjusting precursors for MS2 spectra and spectra masses for MS1 spectra. "
+"It uses an ident file with a threshold field and value to calculate the error and will then choose a shifting mechanism to correct masses throughout the file. "
+"It only works on orbitrap, FT, and TOF data. It is designed to work on mzML files created by msconvert from a single dataset (single run), and with an identification file created using that mzML file. "
+"It does not use any 3rd party (vendor DLL) code. "
+"Recommended Scores and thresholds: MS-GF:SpecEValue,-1e-10 (<1e-10); MyriMatch:MVH,35- (>35); xcorr,3- (>3)" };
+
+SpectrumListPtr filterCreator_lockmassRefiner(const MSData& msd, const string& carg, pwiz::util::IterationListenerRegistry* ilr)
+{
+    const string mzToken("mz=");
+    const string mzNegIonsToken("mzNegIons=");
+    const string toleranceToken("tol=");
+
+    string arg = carg;
+    double lockmassMz = parseKeyValuePair<double>(arg, mzToken, 0);
+    double lockmassMzNegIons = parseKeyValuePair<double>(arg, mzNegIonsToken, lockmassMz); // Optional
+    double lockmassTolerance = parseKeyValuePair<double>(arg, toleranceToken, 1.0);
+    bal::trim(arg);
+    if (!arg.empty())
+        throw runtime_error("[lockmassRefiner] unhandled text remaining in argument string: \"" + arg + "\"");
+
+    if (lockmassMz <= 0 || lockmassTolerance <= 0 || lockmassMzNegIons <= 0)
+    {
+        cerr << "lockmassMz and lockmassTolerance must be postive real numbers" << endl;
+        return SpectrumListPtr();
+    }
+
+    return SpectrumListPtr(new SpectrumList_LockmassRefiner(msd.run.spectrumListPtr, lockmassMz, lockmassMzNegIons, lockmassTolerance));
+}
+UsageInfo usage_lockmassRefiner = { "mz=<real> mzNegIons=<real (mz)> tol=<real (1.0 Daltons)>", "For Waters data, adjusts m/z values according to the specified lockmass m/z and tolerance. Distinct m/z value for negative ions is optional and defaults to the given mz value. For other data, currently does nothing." };
+
+SpectrumListPtr filterCreator_precursorRefine(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     return SpectrumListPtr(new SpectrumList_PrecursorRefine(msd));
 }
@@ -602,7 +793,7 @@ UsageInfo usage_precursorRefine = {"", "This filter recalculates the precursor m
     "It looks at the prior MS1 scan to better infer the parent mass.  It only works on orbitrap, FT, and TOF data. "
     "It does not use any 3rd party (vendor DLL) code."};
 
-SpectrumListPtr filterCreator_mzWindow(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_mzWindow(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     double mzLow = 0;
     double mzHigh = 0;
@@ -625,7 +816,7 @@ UsageInfo usage_mzWindow = {"<mzrange>",
     "100.1 to 307.5, use --filter \"mzWindow [100.1,307.5]\" ."
 };
 
-SpectrumListPtr filterCreator_mzPrecursors(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_mzPrecursors(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     char open='\0', comma='\0', close='\0';
     std::set<double> setMz;
@@ -657,7 +848,7 @@ UsageInfo usage_mzPrecursors = {"<precursor_mz_list>",
     "Note that this filter will drop MS1 scans unless you include 0.0 in the list of precursor values."
     };
 
-SpectrumListPtr filterCreator_msLevel(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_msLevel(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     IntegerSet msLevelSet;
     msLevelSet.parse(arg);
@@ -670,7 +861,7 @@ UsageInfo usage_msLevel = {"<mslevels>",
     "This filter selects only spectra with the indicated <mslevels>, expressed as an int_set."}; 
 
 
-SpectrumListPtr filterCreator_mzPresent(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_mzPresent(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     istringstream parser(arg);
 
@@ -752,7 +943,7 @@ UsageInfo usage_mzPresent = {"<tolerance> <type> <threshold> <orientation> <mz_l
     "used the filter drops data points that match the various criteria instead of keeping them."
 };
 
-SpectrumListPtr filterCreator_chargeState(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_chargeState(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     IntegerSet chargeStateSet;
     chargeStateSet.parse(arg);
@@ -765,7 +956,7 @@ UsageInfo usage_chargeState = {"<charge_states>",
     "This filter keeps spectra that match the listed charge state(s), expressed as an int_set.  Both known/single "
     "and possible/multiple charge states are tested.  Use 0 to include spectra with no charge state at all."};
 
-SpectrumListPtr filterCreator_defaultArrayLength(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_defaultArrayLength(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     IntegerSet defaultArrayLengthSet;
     defaultArrayLengthSet.parse(arg);
@@ -781,7 +972,7 @@ UsageInfo usage_defaultArrayLength = { "<peak_count_range>",
     };
 
 
-SpectrumListPtr filterCreator_metadataFixer(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_metadataFixer(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     return SpectrumListPtr(new SpectrumList_MetadataFixer(msd.run.spectrumListPtr));
 }
@@ -792,7 +983,7 @@ UsageInfo usage_metadataFixer={"","This filter is used to add or replace a spect
     "peak picking for some strange results. Certainly adding up all the samples of profile data to get the TIC is "
     "just wrong, but we do it anyway."};
 
-SpectrumListPtr filterCreator_titleMaker(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_titleMaker(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     return SpectrumListPtr(new SpectrumList_TitleMaker(msd, bal::trim_copy(arg)));
 }
@@ -820,32 +1011,48 @@ UsageInfo usage_titleMaker={"<format_string>","This filter adds or replaces spec
      "--filter \"titleMaker <RunId>.<ScanNumber>.<ScanNumber>.<ChargeState>\""
 };
 
-SpectrumListPtr filterCreator_chargeStatePredictor(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_chargeStatePredictor(const MSData& msd, const string& carg, pwiz::util::IterationListenerRegistry* ilr)
 {
-    istringstream parser(arg);
+    const string overrideExistingChargeToken("overrideExistingCharge=");
+    const string maxMultipleChargeToken("maxMultipleCharge=");
+    const string minMultipleChargeToken("minMultipleCharge=");
+    const string singleChargeFractionTICToken("singleChargeFractionTIC=");
+    const string maxKnownChargeToken("maxKnownCharge=");
+    const string makeMS2Token("makeMS2=");
 
-    string overrideExistingCharge, maxMultipleCharge, minMultipleCharge, singleChargeFractionTIC, makeMS2;
-    parser >> overrideExistingCharge >> maxMultipleCharge >> minMultipleCharge >> singleChargeFractionTIC >> makeMS2;
+    string arg = carg;
+    bool overrideExistingCharge = parseKeyValuePair<bool>(arg, overrideExistingChargeToken, false);
+    int maxMultipleCharge = parseKeyValuePair<int>(arg, maxMultipleChargeToken, 3);
+    int minMultipleCharge = parseKeyValuePair<int>(arg, minMultipleChargeToken, 2);
+    double singleChargeFractionTIC = parseKeyValuePair<double>(arg, singleChargeFractionTICToken, 0.9);
+    int maxKnownCharge = parseKeyValuePair<int>(arg, maxKnownChargeToken, 0);
+    bool makeMS2 = parseKeyValuePair<bool>(arg, makeMS2Token, false);
+    bal::trim(arg);
+    if (!arg.empty())
+        throw runtime_error("[chargeStatePredictor] unhandled text remaining in argument string: \"" + arg + "\"");
 
     return SpectrumListPtr(new
         SpectrumList_ChargeStateCalculator(msd.run.spectrumListPtr,
-                                           overrideExistingCharge == "false" || overrideExistingCharge == "0" ? false : true,
-                                           maxMultipleCharge!=""?lexical_cast<int>(maxMultipleCharge):3,
-                                           minMultipleCharge!=""?lexical_cast<int>(minMultipleCharge):2,
-                                           singleChargeFractionTIC!=""?lexical_cast<double>(singleChargeFractionTIC):0.9,
-                                           makeMS2 == "true" || makeMS2 == "1" ? true : false));
+                                           overrideExistingCharge,
+                                           maxMultipleCharge,
+                                           minMultipleCharge,
+                                           singleChargeFractionTIC,
+                                           maxKnownCharge,
+                                           makeMS2));
 }
-UsageInfo usage_chargeStatePredictor = {"[<overrideExistingCharge> [<maxMultipleCharge> [<minMultipleCharge> [<singleChargeFractionTIC> [<algorithmMakeMS2>]]]]]",
+UsageInfo usage_chargeStatePredictor = {"[overrideExistingCharge=<true|false (false)>] [maxMultipleCharge=<int (3)>] [minMultipleCharge=<int (2)>] [singleChargeFractionTIC=<real (0.9)>] [maxKnownCharge=<int (0)>] [makeMS2=<true|false (false)>]",
     "Predicts MSn spectrum precursors to be singly or multiply charged depending on the ratio of intensity above and below the precursor m/z, or optionally using the \"makeMS2\" algorithm\n"
-    "  <overrideExistingCharge> : always override existing charge information (default:\"true\")\n"
+    "  <overrideExistingCharge> : always override existing charge information (default:\"false\")\n"
     "  <maxMultipleCharge> (default 3) and <minMultipleCharge> (default 2): range of values to add to the spectrum's existing \"MS_possible_charge_state\" values."
     "If these are the same values, the spectrum's MS_possible_charge_state values are removed and replaced with this single value.\n"
     "  <singleChargeFractionTIC> : is a percentage expressed as a value between 0 and 1 (the default is 0.9, or 90 percent). "
-    "This is the value used as the previously mentioned ratio of intensity above and below the precursor m/z.\n" 
+    "This is the value used as the previously mentioned ratio of intensity above and below the precursor m/z.\n"
+    "  <maxKnownCharge> (default is 0, meaning no maximum): the maximum charge allowed for \"known\" charges even if override existing charge is false. "
+    "This allows overriding junk charge calls like +15 peptides.\n"
     "  <algorithmMakeMS2> : default is \"false\", when set to \"true\" the \"makeMS2\" algorithm is used instead of the one described above."
     };
 
-SpectrumListPtr filterCreator_chargeFromIsotope(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_chargeFromIsotope(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
 
     // defaults
@@ -961,7 +1168,7 @@ UsageInfo usage_chargeFromIsotope = {"[minCharge=<minCharge>] [maxCharge=<maxCha
   *   output files containing only ETD or CID MSn data where both activation modes have been
   *   interleaved within a given input vendor data file (eg: Thermo's Decision Tree acquisition mode).
   */ 
-SpectrumListPtr filterCreator_ActivationType(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_ActivationType(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     istringstream parser(arg);
     string activationType;
@@ -1030,7 +1237,7 @@ UsageInfo usage_activation = { "<precursor_activation_type>",
     "   <precursor_activation_type> is any one of: ETD CID SA HCD HECID BIRD ECD IRMPD PD PSD PQD SID or SORI."
     };
 
-SpectrumListPtr filterCreator_AnalyzerType(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_AnalyzerType(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     istringstream parser(arg);
     string analyzerType;
@@ -1067,7 +1274,7 @@ UsageInfo usage_analyzerType = { "<analyzer>",
     "because there are no hybrid FT+Orbi instruments - so this filter does too.\n"
 };
 
-SpectrumListPtr filterCreator_thresholdFilter(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_thresholdFilter(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     istringstream parser(arg);
     string byTypeArg, orientationArg;
@@ -1136,7 +1343,7 @@ UsageInfo usage_thresholdFilter={"<type> <threshold> <orientation> [<mslevels>]"
     "   <mslevels> is an optional int_set of MS levels - if provided, only scans with those MS levels will be filtered, and others left untouched."
 };
 
-SpectrumListPtr filterCreator_polarityFilter(const MSData& msd, const string& arg)
+SpectrumListPtr filterCreator_polarityFilter(const MSData& msd, const string& arg, pwiz::util::IterationListenerRegistry* ilr)
 {
     istringstream parser(arg);
     string polarityArg;
@@ -1177,6 +1384,8 @@ JumpTableEntry jumpTable_[] =
     {"msLevel", usage_msLevel, filterCreator_msLevel},
     {"chargeState", usage_chargeState, filterCreator_chargeState},
     {"precursorRecalculation", usage_precursorRecalculation, filterCreator_precursorRecalculation},
+    {"mzRefiner", usage_mzRefine, filterCreator_mzRefine},
+    {"lockmassRefiner", usage_lockmassRefiner, filterCreator_lockmassRefiner},
     {"precursorRefine", usage_precursorRefine, filterCreator_precursorRefine},
     {"peakPicking", usage_nativeCentroid, filterCreator_nativeCentroid},
     {"scanNumber", usage_scanNumber, filterCreator_scanNumber},
@@ -1225,7 +1434,7 @@ struct HasCommand
 
 
 PWIZ_API_DECL
-void SpectrumListFactory::wrap(MSData& msd, const string& wrapper)
+void SpectrumListFactory::wrap(MSData& msd, const string& wrapper, pwiz::util::IterationListenerRegistry* ilr)
 {
     if (!msd.run.spectrumListPtr)
         return;
@@ -1273,7 +1482,7 @@ void SpectrumListFactory::wrap(MSData& msd, const string& wrapper)
         return;
     }
 
-    SpectrumListPtr filter = entry->creator(msd, arg);
+    SpectrumListPtr filter = entry->creator(msd, arg, ilr);
     msd.filterApplied(); // increase the filter count
 
     if (!filter.get())
@@ -1290,15 +1499,15 @@ void SpectrumListFactory::wrap(MSData& msd, const string& wrapper)
 
 
 PWIZ_API_DECL
-void SpectrumListFactory::wrap(msdata::MSData& msd, const vector<string>& wrappers)
+void SpectrumListFactory::wrap(msdata::MSData& msd, const vector<string>& wrappers, pwiz::util::IterationListenerRegistry* ilr)
 {
     for (vector<string>::const_iterator it=wrappers.begin(); it!=wrappers.end(); ++it)
-        wrap(msd, *it);
+        wrap(msd, *it, ilr);
 }
 
 
 PWIZ_API_DECL
-string SpectrumListFactory::usage(bool detailedHelp,const char *morehelp_prompt)
+string SpectrumListFactory::usage(bool detailedHelp, const char *morehelp_prompt, int maxLineLength)
 {
     ostringstream oss;
     MSData fakemsd;
@@ -1337,9 +1546,8 @@ string SpectrumListFactory::usage(bool detailedHelp,const char *morehelp_prompt)
 
     // tidy up the word wrap
     std::string str = oss.str();
-    const size_t wrap = 70; // wrap at 70 columns
     size_t lastPos = 0;
-    for (size_t curPos = wrap ; curPos < str.length(); ) 
+    for (size_t curPos = maxLineLength; curPos < str.length();)
     {
         std::string::size_type newlinePos = str.rfind( '\n', curPos );
         if( newlinePos == std::string::npos || (newlinePos <= lastPos))
@@ -1353,13 +1561,13 @@ string SpectrumListFactory::usage(bool detailedHelp,const char *morehelp_prompt)
             else 
             {
                 str[ spacePos ] = '\n';
-                curPos = spacePos + wrap + 1;
+                curPos = spacePos + maxLineLength + 1;
             }
         } 
         else 
         {
             lastPos = curPos;
-            curPos = newlinePos + wrap + 1;
+            curPos = newlinePos + maxLineLength + 1;
         }
     }
 
