@@ -1,5 +1,5 @@
 //
-// $Id: SpectrumList_FilterTest.cpp 4352 2013-02-20 09:36:33Z mwilhelm42 $
+// $Id: SpectrumList_FilterTest.cpp 6478 2014-07-08 20:01:38Z chambm $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -66,7 +66,7 @@ SpectrumListPtr createSpectrumList()
         SpectrumPtr spectrum(new Spectrum);
         spectrum->index = i;
         spectrum->id = "scan=" + lexical_cast<string>(100+i);
-        spectrum->setMZIntensityPairs(vector<MZIntensityPair>(i), MS_number_of_counts);
+        spectrum->setMZIntensityPairs(vector<MZIntensityPair>(i), MS_number_of_detector_counts);
 
         // add mz/intensity to the spectra for mzPresent filter
         vector<MZIntensityPair> mzint(i*2);
@@ -74,7 +74,7 @@ SpectrumListPtr createSpectrumList()
         {
             mzint.insert(mzint.end(), MZIntensityPair(j*100, j*j));
         }
-        spectrum->setMZIntensityPairs(mzint, MS_number_of_counts);
+        spectrum->setMZIntensityPairs(mzint, MS_number_of_detector_counts);
 
         bool isMS1 = i%3==0;
         spectrum->set(MS_ms_level, isMS1 ? 1 : 2);
@@ -111,7 +111,7 @@ SpectrumListPtr createSpectrumList()
         }
         else if (i==4) // HCD
         {
-            spectrum->precursors[0].activation.set(MS_high_energy_collision_induced_dissociation);
+            spectrum->precursors[0].activation.set(MS_HCD);
         }
         else if (i==8) // IRMPD
         {
@@ -466,7 +466,7 @@ void testMS2Activation(SpectrumListPtr sl)
     set<CVID> cvIDs;
     // CID
     cvIDs.insert(MS_electron_transfer_dissociation);
-    cvIDs.insert(MS_high_energy_collision_induced_dissociation);
+    cvIDs.insert(MS_HCD);
     cvIDs.insert(MS_IRMPD);
     SpectrumList_Filter filter(ms2filter,
                         SpectrumList_FilterPredicate_ActivationType(cvIDs, true));
@@ -513,7 +513,7 @@ void testMS2Activation(SpectrumListPtr sl)
 
     // HCD
     cvIDs.clear();
-    cvIDs.insert(MS_high_energy_collision_induced_dissociation);
+    cvIDs.insert(MS_HCD);
     SpectrumList_Filter filter3(ms2filter, 
                         SpectrumList_FilterPredicate_ActivationType(cvIDs, false));
     if (os_) 
