@@ -1,5 +1,5 @@
 //
-// $Id: SpectrumList_PeakPicker.cpp 6389 2014-06-13 19:46:19Z frenchwr $
+// $Id: SpectrumList_PeakPicker.cpp 9071 2015-10-29 23:25:16Z pcbrefugee $
 //
 //
 // Original author: Matt Chambers <matt.chambers <a.t> vanderbilt.edu>
@@ -106,7 +106,7 @@ SpectrumList_PeakPicker::SpectrumList_PeakPicker(
         method.userParams.push_back(UserParam("ABI/DataExplorer peak picking"));
     //else
     //    method.userParams.push_back(algorithm->name());
-    if (preferVendorPeakPicking && !mode_)
+    if (preferVendorPeakPicking && !mode_ && (algorithm_ != NULL)) // VendorOnlyPeakPicker sets algorithm null, we deal with this at get binary data time
     {
         cerr << "Warning: vendor peakPicking was requested, but is unavailable";
 #ifdef WIN32
@@ -128,15 +128,8 @@ PWIZ_API_DECL bool SpectrumList_PeakPicker::accept(const msdata::SpectrumListPtr
 
 PWIZ_API_DECL SpectrumPtr SpectrumList_PeakPicker::spectrum(size_t index, DetailLevel detailLevel) const
 {
-
-    // this is useful when a peak-picker is nested inside another filter, so that peak-picking
-    // is not performed twice inadventantly. E.g., the turbocharger constructor iterates over 
-    // all spectra in order to find and sort the ms1 spectra. If a peak-picker was listed as the 
-    // first filter, a spectrum(index,true) call would perform peak-picking when all we are looking
-    // for is the spectrum metadata (i.e., retention time).
-
-    return inner_->spectrum(index,detailLevel);
-
+    // for full metadata, defaultArrayLength must be accurate, so go ahead and do peak picking anyway
+    return (int) detailLevel >= (int) DetailLevel_FullMetadata ? spectrum(index, true) : inner_->spectrum(index, detailLevel);
 }
 
 
@@ -168,11 +161,11 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_PeakPicker::spectrum(size_t index, bool g
 
         case 0:
         default:
-            s = inner_->spectrum(index, true);
+            s = inner_->spectrum(index, true); // TODO you'd think this would be "getBinaryData" instead of "true" but that breaks SpectrumListFactoryTest
             break;
     }
 
-    if (!msLevelsToPeakPick_.contains(s->cvParam(MS_ms_level).valueAs<int>()))
+    if (!getBinaryData || !msLevelsToPeakPick_.contains(s->cvParam(MS_ms_level).valueAs<int>()))
         return s;
 
     vector<CVParam>& cvParams = s->cvParams;
@@ -208,6 +201,8 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_PeakPicker::spectrum(size_t index, bool g
 
     try
     {
+        if (algorithm_ == NULL) // As with VendorOnlyPeakPicker
+            throw NoVendorPeakPickingException();
         vector<double>& mzs = s->getMZArray()->data;
         vector<double>& intensities = s->getIntensityArray()->data;
         vector<double> xPeakValues, yPeakValues;
