@@ -1,5 +1,5 @@
 //
-// $Id: obo.cpp 6478 2014-07-08 20:01:38Z chambm $
+// $Id: obo.cpp 6909 2014-11-19 17:18:29Z chambm $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -24,8 +24,9 @@
 #define PWIZ_SOURCE
 
 #include "obo.hpp"
-#include "boost/regex.hpp"
+#include "boost/xpressive/xpressive_dynamic.hpp"
 #include "pwiz/utility/misc/Std.hpp"
+namespace bxp = boost::xpressive;
 
 namespace pwiz {
 namespace data {
@@ -46,7 +47,7 @@ namespace {
 // line into [tag, value, dbxrefs, comment] if the OBO grammar were more regular.
 
 
-// boost::regex notes:
+// regex notes:
 //  parentheses are used for submatch captures
 //  \\d matches digit
 //  \\s matches whitespace 
@@ -90,10 +91,10 @@ istream& getcleanline(istream& is, string& buffer)
 
 void parse_id(const string& line, Term& term)
 {
-    static const boost::regex e("id:\\s*(\\w+):(\\d+)\\s*");
+    static const bxp::sregex e = bxp::sregex::compile("id:\\s*(\\w+):(\\d+)\\s*");
 
-    boost::smatch what; 
-    if (!regex_match(line, what, e))
+    bxp::smatch what; 
+    if (!bxp::regex_match(line, what, e))
         throw runtime_error("Error matching term id on line: \"" + line + "\"");
 
     term.prefix = unescape_copy(what[1]);
@@ -108,10 +109,10 @@ void parse_id(const string& line, Term& term)
 
 void parse_name(const string& line, Term& term)
 {
-    static const boost::regex e("name:\\s*(.*?)\\s*");
+    static const bxp::sregex e = bxp::sregex::compile("name:\\s*(.*?)\\s*");
 
-    boost::smatch what; 
-    if (!regex_match(line, what, e))
+    bxp::smatch what; 
+    if (!bxp::regex_match(line, what, e))
         throw runtime_error("Error matching term name on line: \"" + line + "\"");    
 
     term.name = unescape_copy(what[1]);
@@ -120,10 +121,10 @@ void parse_name(const string& line, Term& term)
 
 void parse_def(const string& line, Term& term)
 {
-    static const boost::regex e("def:\\s*\"(OBSOLETE )?(.*)\"\\s*\\[.*\\].*");
+    static const bxp::sregex e = bxp::sregex::compile("def:\\s*\"(OBSOLETE )?(.*)\"\\s*\\[.*\\].*");
 
-    boost::smatch what; 
-    if (!regex_match(line, what, e))
+    bxp::smatch what; 
+    if (!bxp::regex_match(line, what, e))
         throw runtime_error("Error matching term def on line: \"" + line + "\"");
 
     term.def = what[2];
@@ -133,15 +134,15 @@ void parse_def(const string& line, Term& term)
 
 void parse_relationship(const string& line, Term& term)
 {
-    static const boost::regex e("relationship:\\s*(\\w+) (\\w+):(\\d+).*");
+    static const bxp::sregex e = bxp::sregex::compile("relationship:\\s*(\\w+) (\\w+):(\\d+).*");
 
-    boost::smatch what; 
-    if (!regex_match(line, what, e))
+    bxp::smatch what; 
+    if (!bxp::regex_match(line, what, e))
         throw runtime_error("Error matching term relationship on line: \"" + line + "\"");
 
     if (what[1] == "part_of")
     {
-        if (what[2] != term.prefix)
+        if (what[2].str() != term.prefix)
             cerr << "[obo] Ignoring part_of relationship with different prefix:\n  " << line << endl;
         else
             term.parentsPartOf.push_back(lexical_cast<Term::id_type>(what[3]));
@@ -154,10 +155,10 @@ void parse_relationship(const string& line, Term& term)
 
 void parse_is_obsolete(const string& line, Term& term)
 {
-    static const boost::regex e("is_obsolete:\\s*(\\w+)\\s*");
+    static const bxp::sregex e = bxp::sregex::compile("is_obsolete:\\s*(\\w+)\\s*");
 
-    boost::smatch what; 
-    if (!regex_match(line, what, e))
+    bxp::smatch what; 
+    if (!bxp::regex_match(line, what, e))
         throw runtime_error("Error matching term is_obsolete on line: \"" + line + "\"");    
 
     term.isObsolete = (what[1] == "true");
@@ -166,13 +167,13 @@ void parse_is_obsolete(const string& line, Term& term)
 
 void parse_is_a(const string& line, Term& term)
 {
-    static const boost::regex e("is_a:\\s*(\\w+):(\\d+).*");
+    static const bxp::sregex e = bxp::sregex::compile("is_a:\\s*(\\w+):(\\d+).*");
 
-    boost::smatch what; 
-    if (!regex_match(line, what, e))
+    bxp::smatch what; 
+    if (!bxp::regex_match(line, what, e))
         throw runtime_error("Error matching term is_a on line: \"" + line + "\"");    
 
-    if (what[1] != term.prefix)
+    if (what[1].str() != term.prefix)
     {
         cerr << "[obo] Ignoring is_a with different prefix:\n  " << line << endl;
         return;
@@ -184,10 +185,10 @@ void parse_is_a(const string& line, Term& term)
 
 void parse_exact_synonym(const string& line, Term& term)
 {
-    static const boost::regex e("exact_synonym:\\s*\"(.*)\".*");
+    static const bxp::sregex e = bxp::sregex::compile("exact_synonym:\\s*\"(.*)\".*");
 
-    boost::smatch what; 
-    if (!regex_match(line, what, e))
+    bxp::smatch what; 
+    if (!bxp::regex_match(line, what, e))
         throw runtime_error("Error matching term exact_synonym on line: \"" + line + "\"");    
 
     term.exactSynonyms.push_back(unescape_copy(what[1]));
@@ -196,10 +197,10 @@ void parse_exact_synonym(const string& line, Term& term)
 
 void parse_synonym(const string& line, Term& term)
 {
-    static const boost::regex e("synonym:\\s*\"(.*)\"\\s*(\\w+)?.*");
+    static const bxp::sregex e = bxp::sregex::compile("synonym:\\s*\"(.*)\"\\s*(\\w+)?.*");
 
-    boost::smatch what; 
-    if (!regex_match(line, what, e))
+    bxp::smatch what; 
+    if (!bxp::regex_match(line, what, e))
         throw runtime_error("Error matching term synonym on line: \"" + line + "\"");    
 
     if (what[2] == "EXACT")
@@ -209,10 +210,10 @@ void parse_synonym(const string& line, Term& term)
 
 void parse_property_value(const string& line, Term& term)
 {
-    static const boost::regex e("property_value:\\s*(\\S+?)\\s*[=:]?\\s*\"(.*)\".*");
+    static const bxp::sregex e = bxp::sregex::compile("property_value:\\s*(\\S+?)\\s*[=:]?\\s*\"(.*)\".*");
 
-    boost::smatch what;
-    if (!regex_match(line, what, e))
+    bxp::smatch what;
+    if (!bxp::regex_match(line, what, e))
         throw runtime_error("Error matching term property_value on line: \"" + line + "\"");
 
     term.propertyValues.insert(make_pair(what[1], what[2]));
@@ -221,10 +222,10 @@ void parse_property_value(const string& line, Term& term)
 
 void parse_xref(const string& line, Term& term)
 {
-    static const boost::regex e("xref:\\s*(\\S+?)\\s*\"(.*)\".*");
+    static const bxp::sregex e = bxp::sregex::compile("xref:\\s*(\\S+?)\\s*\"(.*)\".*");
 
-    boost::smatch what;
-    if (!regex_match(line, what, e))
+    bxp::smatch what;
+    if (!bxp::regex_match(line, what, e))
         throw runtime_error("Error matching term xref on line: \"" + line + "\"");
 
     term.propertyValues.insert(make_pair(unescape_copy(what[1]), unescape_copy(what[2])));
