@@ -1,5 +1,5 @@
 //
-// $Id: DiffTest.cpp 4129 2012-11-20 00:05:37Z chambm $
+// $Id: DiffTest.cpp 5759 2014-02-19 22:26:29Z chambm $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -105,7 +105,7 @@ void testFileDescription()
     SourceFilePtr source1(new SourceFile("id1"));
     SourceFilePtr source2a(new SourceFile("id2"));
     SourceFilePtr source2b(new SourceFile("id2"));
-    source2a->cvParams.push_back(MS_Thermo_RAW_file);
+    source2a->cvParams.push_back(MS_Thermo_RAW_format);
 
     a.sourceFilePtrs.push_back(source1);
     b.sourceFilePtrs.push_back(source1);
@@ -129,9 +129,9 @@ void testFileDescription()
     unit_assert(diff.b_a.contacts[0].cvParam(MS_contact_name).value == "Isabelle Lynn");
 
     unit_assert(diff.a_b.sourceFilePtrs.size() == 1);
-    unit_assert(diff.a_b.sourceFilePtrs[0]->hasCVParam(MS_Thermo_RAW_file));
+    unit_assert(diff.a_b.sourceFilePtrs[0]->hasCVParam(MS_Thermo_RAW_format));
     unit_assert(diff.b_a.sourceFilePtrs.size() == 1);
-    unit_assert(!diff.b_a.sourceFilePtrs[0]->hasCVParam(MS_Thermo_RAW_file));
+    unit_assert(!diff.b_a.sourceFilePtrs[0]->hasCVParam(MS_Thermo_RAW_format));
 }
 
 
