@@ -1,5 +1,5 @@
 //
-// $Id: IO.cpp 6585 2014-08-07 22:49:28Z chambm $
+// $Id: IO.cpp 8934 2015-10-06 22:03:06Z pcbrefugee $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -1661,12 +1661,15 @@ void write(minimxml::XMLWriter& writer, const BinaryDataArray& binaryDataArray,
     }
     switch (usedConfig.numpress) {
         case BinaryDataEncoder::Numpress_Linear:
+            write(writer, MS_32_bit_float); // This should actually be ignored by any reader since numpress defines word size and format, but it makes output standards-compliant and is pretty close to true anyway
             write(writer, MS_MS_Numpress_linear_prediction_compression);
             break;
         case BinaryDataEncoder::Numpress_Pic:
+            write(writer, MS_32_bit_integer); // This should actually be ignored by any reader since numpress defines word size and format, but it makes output standards-compliant and is pretty close to true anyway
             write(writer, MS_MS_Numpress_positive_integer_compression);
             break;
         case BinaryDataEncoder::Numpress_Slof:
+            write(writer, MS_32_bit_float); // This should actually be ignored by any reader since numpress defines word size and format, but it makes output standards-compliant and is pretty close to true anyway
             write(writer, MS_MS_Numpress_short_logged_float_compression);
             break;
         case BinaryDataEncoder::Numpress_None:
@@ -1854,11 +1857,13 @@ struct HandlerBinaryDataArray : public HandlerParamContainer
         switch (cvidBinaryDataType)
         {
             case MS_32_bit_float:
-                config.precision = BinaryDataEncoder::Precision_32;
+                if (BinaryDataEncoder::Numpress_None == config.numpress)
+                    config.precision = BinaryDataEncoder::Precision_32;
                 break;
             case MS_64_bit_float:
                 config.precision = BinaryDataEncoder::Precision_64;
                 break;
+            case MS_32_bit_integer:
             case CVID_Unknown:
                 if (BinaryDataEncoder::Numpress_None == config.numpress) // 32 vs 64 bit is meaningless in numpress
                     throw runtime_error("[IO::HandlerBinaryDataArray] Missing binary data type.");
