@@ -1,5 +1,5 @@
 //
-// $Id: Reader.cpp 6239 2014-05-24 03:37:45Z chambm $
+// $Id: Reader.cpp 9283 2016-01-11 21:31:03Z chambm $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -41,6 +41,8 @@ Reader::Config::Config()
     , srmAsSpectra(false)
     , acceptZeroLengthSpectra(false)
     , combineIonMobilitySpectra(false)
+    , unknownInstrumentIsError(false)
+    , adjustUnknownTimeZonesToHostTimeZone(true)
 {
 }
 
@@ -51,6 +53,8 @@ Reader::Config::Config(const Config& rhs)
     srmAsSpectra = rhs.srmAsSpectra;
 	acceptZeroLengthSpectra = rhs.acceptZeroLengthSpectra;
     combineIonMobilitySpectra = rhs.combineIonMobilitySpectra;
+    unknownInstrumentIsError = rhs.unknownInstrumentIsError;
+    adjustUnknownTimeZonesToHostTimeZone = rhs.adjustUnknownTimeZonesToHostTimeZone;
 }
 
 // default implementation; most Readers don't need to worry about multi-run input files
@@ -191,7 +195,7 @@ PWIZ_API_DECL CVID identifyFileFormat(const ReaderPtr& reader, const std::string
         else if (type == "Mascot Generic") return MS_Mascot_MGF_format;
         else if (type == "MSn") return MS_MS2_format;
         else if (type == "ABSciex WIFF") return MS_ABI_WIFF_format;
-        else if (type == "ABSciex T2D") return MS_AB_SCIEX_TOF_TOF_T2D_format;
+        else if (type == "ABSciex T2D") return MS_SCIEX_TOF_TOF_T2D_format;
         else if (type == "Agilent MassHunter") return MS_Agilent_MassHunter_format;
         else if (type == "Thermo RAW") return MS_Thermo_RAW_format;
         else if (type == "Waters RAW") return MS_Waters_raw_format;
