@@ -1,5 +1,5 @@
 //
-// $Id: Reader.hpp 6239 2014-05-24 03:37:45Z chambm $
+// $Id: Reader.hpp 8904 2015-09-29 19:20:14Z chambm $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -52,6 +52,13 @@ class PWIZ_API_DECL Reader
         /// when true, all drift bins/scans in a frame/block are written in combined form instead of as individual spectra
         bool combineIonMobilitySpectra;
 
+        /// when true, if a reader cannot identify an instrument, an exception will be thrown asking users to report it
+        bool unknownInstrumentIsError;
+
+        /// when true, if a reader does not know what time zone was used to record a time, it will assume the time refers to the host's local time;
+        /// when false, the reader will treat times with unknown time zone as UTC
+        bool adjustUnknownTimeZonesToHostTimeZone;
+
         Config();
         Config(const Config& rhs);
     };
