@@ -1,5 +1,5 @@
 //
-// $Id: SpectrumList_mzML.cpp 6585 2014-08-07 22:49:28Z chambm $
+// $Id: SpectrumList_mzML.cpp 8725 2015-08-04 15:49:57Z pcbrefugee $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -56,6 +56,7 @@ class SpectrumList_mzMLImpl : public SpectrumList_mzML
     virtual size_t find(const std::string& id) const;
     virtual IndexList findSpotID(const std::string& spotID) const;
     virtual SpectrumPtr spectrum(size_t index, bool getBinaryData) const;
+    virtual SpectrumPtr spectrum(size_t index, DetailLevel detailLevel) const;
     virtual SpectrumPtr spectrum(const SpectrumPtr &seed, bool getBinaryData) const;
     virtual SpectrumPtr spectrum(size_t index, IO::BinaryDataFlag binaryDataFlag, const SpectrumPtr *defaults) const;
 
@@ -107,6 +108,11 @@ IndexList SpectrumList_mzMLImpl::findSpotID(const string& spotID) const
     return index_->findSpectrumBySpotID(spotID);
 }
 
+SpectrumPtr SpectrumList_mzMLImpl::spectrum(size_t index, DetailLevel detailLevel) const
+{
+    return spectrum(index, (detailLevel == DetailLevel_FullData) ? IO::ReadBinaryData : IO::IgnoreBinaryData, NULL);
+}
+
 SpectrumPtr SpectrumList_mzMLImpl::spectrum(size_t index, bool getBinaryData) const
 {
     return spectrum(index, getBinaryData ? IO::ReadBinaryData : IO::IgnoreBinaryData, NULL);
