@@ -1,5 +1,5 @@
 //
-// $Id: SpectrumList_mzXML.cpp 4129 2012-11-20 00:05:37Z chambm $
+// $Id: SpectrumList_mzXML.cpp 6585 2014-08-07 22:49:28Z chambm $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -27,6 +27,7 @@
 #include "References.hpp"
 #include "pwiz/utility/minimxml/SAXParser.hpp"
 #include "pwiz/utility/misc/Std.hpp"
+#include <boost/thread.hpp>
 
 
 namespace pwiz {
@@ -50,15 +51,18 @@ class SpectrumList_mzXMLImpl : public SpectrumList_mzXML
     virtual size_t size() const {return index_.size();}
     virtual const SpectrumIdentity& spectrumIdentity(size_t index) const;
     virtual size_t find(const string& id) const;
+    virtual SpectrumPtr spectrum(size_t index, DetailLevel detailLevel) const;
     virtual SpectrumPtr spectrum(size_t index, bool getBinaryData) const;
     virtual SpectrumPtr spectrum(const SpectrumPtr &seed, bool getBinaryData) const;
     virtual SpectrumPtr spectrum(size_t index, IO::BinaryDataFlag binaryDataFlag, const SpectrumPtr *defaults) const;
 
     private:
+    SpectrumPtr spectrum(size_t index, IO::BinaryDataFlag binaryDataFlag, DetailLevel detailLevel, const SpectrumPtr *defaults, bool isRecursiveCall) const;
     shared_ptr<istream> is_;
     const MSData& msd_;
     vector<SpectrumIdentityFromMzXML> index_;
     map<string,size_t> idToIndex_;
+    mutable boost::recursive_mutex readMutex;
 
     mutable vector<int> scanMsLevelCache_;
 
@@ -141,7 +145,7 @@ struct HandlerPrecursor : public SAXParser::Handler
             precursor->selectedIons.push_back(SelectedIon());
 
             if (!precursorIntensity.empty() && precursorIntensity != "0")
-                precursor->selectedIons.back().set(MS_peak_intensity, precursorIntensity, MS_number_of_counts);
+                precursor->selectedIons.back().set(MS_peak_intensity, precursorIntensity, MS_number_of_detector_counts);
 
             if (!precursorCharge.empty())
                 precursor->selectedIons.back().set(MS_charge_state, precursorCharge);
@@ -264,7 +268,7 @@ class HandlerPeaks : public SAXParser::Handler
     {
         if (peaksCount == 0)
         {
-            spectrum_.setMZIntensityArrays(vector<double>(), vector<double>(), MS_number_of_counts);
+            spectrum_.setMZIntensityArrays(vector<double>(), vector<double>(), MS_number_of_detector_counts);
             return Status::Ok;
         }
 
@@ -276,7 +280,7 @@ class HandlerPeaks : public SAXParser::Handler
             throw runtime_error("[SpectrumList_mzXML::HandlerPeaks] Invalid peak count."); 
 
         spectrum_.setMZIntensityPairs(reinterpret_cast<const MZIntensityPair*>(&decoded[0]),
-                                      peaksCount, MS_number_of_counts);
+                                      peaksCount, MS_number_of_detector_counts);
         return Status::Ok;
     }
  
@@ -393,7 +397,6 @@ class HandlerScan : public SAXParser::Handler
             if (scanType.empty() || scanType == "full")
             {
                 spectrum_.set(msLevel == "1" ? MS_MS1_spectrum : MS_MSn_spectrum);
-                scan.set(MS_full_scan);
             } else if (scanType == "zoom")
             {
                 spectrum_.set(MS_MSn_spectrum);
@@ -404,22 +407,23 @@ class HandlerScan : public SAXParser::Handler
                 scan.set(MS_SIM);
             } else if (scanType == "srm" ||
                        scanType == "mrm" || // hack: mzWiff (ABI) and wolf-mrm (Waters) use this value
-                       scanType == "multiplereaction") // hack: Trapper (Agilent) uses this value
+                       scanType == "multiplereaction" || // hack: Trapper (Agilent) uses this value
+                       scanType == "srm_ionprep") // hack: (Bruker) uses this value
             {
                 spectrum_.set(MS_SRM_spectrum);
                 scan.set(MS_SRM);
             } else if (scanType == "crm")
             {
                 spectrum_.set(MS_CRM_spectrum);
-                scan.set(MS_CRM);
+                scan.set(MS_CRM_OBSOLETE);
             } else if (scanType == "q1")
             {
                 spectrum_.set(MS_precursor_ion_spectrum);
-                scan.set(MS_precursor_ion_scan);
+                scan.set(MS_precursor_ion_spectrum);
             } else if (scanType == "q3")
             {
                 spectrum_.set(MS_product_ion_spectrum);
-                scan.set(MS_product_ion_scan);
+                scan.set(MS_product_ion_spectrum);
             }
 
             // TODO: make this more robust
@@ -487,7 +491,7 @@ class HandlerScan : public SAXParser::Handler
 
             if (!getBinaryData_ || handlerPeaks_.peaksCount == 0)
             {
-                spectrum_.setMZIntensityArrays(vector<double>(), vector<double>(), MS_number_of_counts);
+                spectrum_.setMZIntensityArrays(vector<double>(), vector<double>(), MS_number_of_detector_counts);
                 spectrum_.defaultArrayLength = handlerPeaks_.peaksCount;
                 return Status::Ok;
             }
@@ -539,21 +543,52 @@ class HandlerScan : public SAXParser::Handler
     CVID nativeIdFormat_;
 };
 
+/// retrieve a spectrum by index
+/// - detailLevel determines what fields are guaranteed present on the spectrum after the call
+/// - client may assume the underlying Spectrum* is valid 
+SpectrumPtr SpectrumList_mzXMLImpl::spectrum(size_t index, DetailLevel detailLevel) const
+{
+    return spectrum(index,(detailLevel == DetailLevel_FullData) ? IO::ReadBinaryData : IO::IgnoreBinaryData, detailLevel, NULL, false);
+}
+
 
 SpectrumPtr SpectrumList_mzXMLImpl::spectrum(size_t index, bool getBinaryData) const
 {
-    return spectrum(index, getBinaryData ? IO::ReadBinaryData : IO::IgnoreBinaryData, NULL);
+    return spectrum(index, getBinaryData ? IO::ReadBinaryData : IO::IgnoreBinaryData,
+        getBinaryData ? DetailLevel_FullData : DetailLevel_FullMetadata,  NULL, false);
 }
 
 /// get a copy of the seed spectrum with its binary data populated
 /// this is useful for formats like mzXML that can delay loading of binary data
 /// - client may assume the underlying Spectrum* is valid 
 SpectrumPtr SpectrumList_mzXMLImpl::spectrum(const SpectrumPtr &seed, bool getBinaryData) const {
-    return spectrum(seed->index, getBinaryData ? IO::ReadBinaryDataOnly: IO::IgnoreBinaryData, &seed);
+    return spectrum(seed->index, getBinaryData ? IO::ReadBinaryDataOnly: IO::IgnoreBinaryData, 
+        DetailLevel_InstantMetadata, // assume full metadata is already loaded
+        &seed, false);
 }
 
 SpectrumPtr SpectrumList_mzXMLImpl::spectrum(size_t index, IO::BinaryDataFlag binaryDataFlag, const SpectrumPtr *defaults) const
 {
+    DetailLevel detailLevel;
+    switch (binaryDataFlag)
+    {
+    case IO::IgnoreBinaryData:
+        detailLevel = DetailLevel_FullMetadata;
+        break;
+    case IO::ReadBinaryDataOnly:
+        detailLevel = DetailLevel_InstantMetadata;
+        break;
+    case IO::ReadBinaryData:
+    default:
+        detailLevel = DetailLevel_FullData;
+        break;
+    }
+    return spectrum(index, binaryDataFlag, detailLevel, defaults, false);
+}
+
+SpectrumPtr SpectrumList_mzXMLImpl::spectrum(size_t index, IO::BinaryDataFlag binaryDataFlag, DetailLevel detailLevel, const SpectrumPtr *defaults, bool isRecursiveCall) const
+{
+    boost::lock_guard<boost::recursive_mutex> lock(readMutex);  // lock_guard will unlock mutex when out of scope or when exception thrown (during destruction)
     if (index > index_.size())
         throw runtime_error("[SpectrumList_mzXML::spectrum()] Index out of bounds.");
 
@@ -596,19 +631,22 @@ SpectrumPtr SpectrumList_mzXMLImpl::spectrum(size_t index, IO::BinaryDataFlag bi
     int msLevel = result->cvParam(MS_ms_level).valueAs<int>();
     scanMsLevelCache_[index] = msLevel;
 
-    // hack to get parent scanNumber if precursorScanNum wasn't set
-
-    if (msLevel > 1 &&
-        !result->precursors.empty() &&
-        result->precursors.front().spectrumID.empty())
+    if (detailLevel >= DetailLevel_FullMetadata)
     {
-        // MCC: I see your hack and I raise you a hack!
-        // * precursorScanNum is optional
-        // * the precursor scan is not necessarily in the mzXML
-        if (result->precursors.front().spectrumID == "0")
-            result->precursors.front().spectrumID.clear();
-        else
-            result->precursors.front().spectrumID = getPrecursorID(msLevel-1, index);
+        // hack to get parent scanNumber if precursorScanNum wasn't set
+        if (msLevel > 1 &&
+            !isRecursiveCall && // in an all-MS2 file we can run out of stack
+            !result->precursors.empty() &&
+            result->precursors.front().spectrumID.empty())
+        {
+            // MCC: I see your hack and I raise you a hack!
+            // * precursorScanNum is optional
+            // * the precursor scan is not necessarily in the mzXML
+            if (result->precursors.front().spectrumID == "0")
+                result->precursors.front().spectrumID.clear();
+            else
+                result->precursors.front().spectrumID = getPrecursorID(msLevel-1, index);
+        }
     }
 
     // resolve any references into the MSData object
@@ -856,12 +894,13 @@ string SpectrumList_mzXMLImpl::getPrecursorID(int precursorMsLevel, size_t index
         if (index && (cachedMsLevel == 0))
         {
             // populate the missing MS level
-            SpectrumPtr s = spectrum(index, false);
+            SpectrumPtr s = spectrum(index, DetailLevel_FastMetadata); // avoid excessive recursion
+
             cachedMsLevel = s->cvParam(MS_ms_level).valueAs<int>();
         }
         if (cachedMsLevel == precursorMsLevel) 
         {
-            SpectrumPtr s = spectrum(index,false);
+            SpectrumPtr s = spectrum(index, DetailLevel_FastMetadata);
             return s ?  s->id : lexical_cast<string>(index);
         }
     }
