@@ -1,5 +1,5 @@
 //
-// $Id: examples.cpp 5759 2014-02-19 22:26:29Z chambm $ 
+// $Id: examples.cpp 7331 2015-03-24 16:11:12Z chambm $ 
 //
 //
 // Original author: Matt Chambers <matt.chambers .@. vanderbilt.edu>
@@ -112,7 +112,7 @@ PWIZ_API_DECL void addMIAPEExampleMetadata(TraData& td)
     proteinPtr->set(MS_protein_accession, "Q123");
     proteinPtr->sequence = "ABCD";
     proteinPtr->set(MS_protein_name, "A short protein.");
-    proteinPtr->set(MS_molecular_mass, 12345, UO_dalton);
+    proteinPtr->set(MS_molecular_mass_OBSOLETE, 12345, UO_dalton);
     td.proteinPtrs.push_back(proteinPtr);
 
 
