@@ -1,5 +1,5 @@
 //
-// $Id: SpectrumList_ABI.cpp 6585 2014-08-07 22:49:28Z chambm $
+// $Id: SpectrumList_ABI.cpp 8693 2015-07-27 22:01:28Z brendanx $
 //
 //
 // Original author: Matt Chambers <matt.chambers .@. vanderbilt.edu>
@@ -147,13 +147,6 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_ABI::spectrum(size_t index, DetailLevel d
     result->set(translateAsSpectrumType(experimentType));
     result->set(translate(msExperiment->getPolarity()));
 
-    // CONSIDER: Can anything below here be added to instant?
-    if (detailLevel == DetailLevel_InstantMetadata)
-        return result;
-
-	// Revert to previous behavior for getting binary data or not.
-	bool getBinaryData = (detailLevel == DetailLevel_FullData);
-
     double startMz, stopMz;
     msExperiment->getAcquisitionMassRange(startMz, stopMz);
     scan.scanWindows.push_back(ScanWindow(startMz, stopMz, MS_m_z));
@@ -203,6 +196,16 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_ABI::spectrum(size_t index, DetailLevel d
             spectrum->getPrecursorInfo(selectedMz, intensity, charge);
 
             Precursor precursor;
+            if (spectrum->getHasIsolationInfo())
+            {
+                double centerMz, lowerLimit, upperLimit;
+                spectrum->getIsolationInfo(centerMz, lowerLimit, upperLimit);
+                precursor.isolationWindow.set(MS_isolation_window_target_m_z, centerMz, MS_m_z);
+                precursor.isolationWindow.set(MS_isolation_window_lower_offset, centerMz - lowerLimit, MS_m_z);
+                precursor.isolationWindow.set(MS_isolation_window_upper_offset, upperLimit - centerMz, MS_m_z);
+				selectedMz = centerMz;
+            }
+
             SelectedIon selectedIon;
 
             selectedIon.set(MS_selected_ion_m_z, selectedMz, MS_m_z);
@@ -215,10 +218,21 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_ABI::spectrum(size_t index, DetailLevel d
             result->precursors.push_back(precursor);
         }
 
+    if (detailLevel == DetailLevel_InstantMetadata)
+        return result;
+
+    // Revert to previous behavior for getting binary data or not.
+    bool getBinaryData = (detailLevel == DetailLevel_FullData);
+
         //result->set(MS_lowest_observed_m_z, spectrum->getMinX(), MS_m_z);
         //result->set(MS_highest_observed_m_z, spectrum->getMaxX(), MS_m_z);
-        result->set(MS_base_peak_intensity, spectrum->getBasePeakY(), MS_number_of_detector_counts);
-        result->set(MS_base_peak_m_z, spectrum->getBasePeakX(), MS_m_z);
+
+        if (!config_.acceptZeroLengthSpectra)
+        {
+            result->set(MS_base_peak_intensity, spectrum->getBasePeakY(), MS_number_of_detector_counts);
+            result->set(MS_base_peak_m_z, spectrum->getBasePeakX(), MS_m_z);
+        }
+
         result->set(MS_total_ion_current, spectrum->getSumY(), MS_number_of_detector_counts);
 
         if (getBinaryData)
@@ -232,6 +246,8 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_ABI::spectrum(size_t index, DetailLevel d
                 result->set(MS_profile_spectrum); // let SpectrumList_PeakPicker know this was a profile spectrum
         }
 
+        // This forces the WIFF reader to get the data, making full metadata
+        // nearly equivalent in performance to getting binary.
         result->defaultArrayLength = spectrum->getDataSize(doCentroid);
     }
 
@@ -245,6 +261,7 @@ PWIZ_API_DECL void SpectrumList_ABI::createIndex() const
 
     typedef multimap<double, pair<ExperimentPtr, int> > ExperimentAndCycleByTime;
     ExperimentAndCycleByTime experimentAndCycleByTime;
+    vector<double> times, intensities;
 
     int periodCount = wifffile_->getPeriodCount(sample);
     for (int period=1; period <= periodCount; ++period)
@@ -259,12 +276,22 @@ PWIZ_API_DECL void SpectrumList_ABI::createIndex() const
                 (experimentType == SIM && !config_.simAsSpectra))
                 continue;
 
-            vector<double> times, intensities;
-            msExperiment->getBPC(times, intensities);
+            if (config_.acceptZeroLengthSpectra)
+            {
+                msExperiment->getTIC(times, intensities);
 
-            for (int i=0; i < (int) times.size(); ++i)
-                if (intensities[i] > 0 && (config_.acceptZeroLengthSpectra || (wifffile_->getSpectrum(msExperiment, i+1)->getDataSize(false) > 0)))
-                    experimentAndCycleByTime.insert(make_pair(times[i], make_pair(msExperiment, i+1)));
+                for (int i = 0, end = (int) times.size(); i < end; ++i)
+                    if (intensities[i] > 0)
+                        experimentAndCycleByTime.insert(make_pair(times[i], make_pair(msExperiment, i + 1)));
+            }
+            else
+            {
+                msExperiment->getBPC(times, intensities);
+
+                for (int i = 0, end = (int)times.size(); i < end; ++i)
+                    if (intensities[i] > 0 && wifffile_->getSpectrum(msExperiment, i + 1)->getDataSize(false) > 0)
+                        experimentAndCycleByTime.insert(make_pair(times[i], make_pair(msExperiment, i + 1)));
+            }
         }
     }
 
