@@ -1,5 +1,5 @@
 //
-// $Id: SpectrumList_Agilent.cpp 6585 2014-08-07 22:49:28Z chambm $
+// $Id: SpectrumList_Agilent.cpp 7485 2015-05-18 16:05:23Z chambm $
 //
 //
 // Original author: Matt Chambers <matt.chambers .@. vanderbilt.edu>
@@ -431,7 +431,7 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Agilent::spectrum(size_t index, DetailLev
             // Agilent profile mode data returns all zero-intensity samples, so we filter out
             // samples that aren't adjacent to a non-zero-intensity sample value.
 
-            double lowestObservedMz = 0, highestObservedMz = 0;
+            double lowestObservedMz = numeric_limits<double>::max(), highestObservedMz = 0;
             result->defaultArrayLength = 0;
 
             // special case for the first sample
