@@ -1,5 +1,5 @@
 //
-// $Id: Reader_Bruker.cpp 6478 2014-07-08 20:01:38Z chambm $
+// $Id: Reader_Bruker.cpp 6979 2014-12-09 16:15:35Z chambm $
 //
 //
 // Original author: Matt Chambers <matt.chambers .@. vanderbilt.edu>
@@ -92,7 +92,8 @@ void initializeInstrumentConfigurationPtrs(MSData& msd, CompassDataPtr rawfile,
     commonInstrumentParams->id = "CommonInstrumentParams";
     msd.paramGroupPtrs.push_back(commonInstrumentParams);
 
-    commonInstrumentParams->userParams.push_back(UserParam("instrument model", rawfile->getInstrumentDescription()));
+    if (!rawfile->getInstrumentDescription().empty())
+        commonInstrumentParams->userParams.push_back(UserParam("instrument model", rawfile->getInstrumentDescription()));
     commonInstrumentParams->set(cvidSeries);
 
     // create instrument configuration templates based on the instrument model
@@ -128,7 +129,7 @@ void fillInMetadata(const bfs::path& rootpath, MSData& msd, Reader_Bruker_Format
     CVID acquisitionSoftwareCvid = translateAsAcquisitionSoftware(compassDataPtr);
     acquisitionSoftware->id = cvTermInfo(acquisitionSoftwareCvid).shortName();
     acquisitionSoftware->set(acquisitionSoftwareCvid);
-    acquisitionSoftware->version = "unknown";
+    acquisitionSoftware->version = compassDataPtr->getAcquisitionSoftwareVersion();
     msd.softwarePtrs.push_back(acquisitionSoftware);
 
     SoftwarePtr softwarePwiz(new Software);
