@@ -1,5 +1,5 @@
 //
-// $Id: Reader_Thermo_Detail.cpp 6478 2014-07-08 20:01:38Z chambm $
+// $Id: Reader_Thermo_Detail.cpp 7303 2015-03-13 20:19:40Z chambm $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -94,6 +94,7 @@ PWIZ_API_DECL CVID translateAsInstrumentModel(InstrumentModelType instrumentMode
         case InstrumentModelType_TSQ_Quantum_Ultra:         return MS_TSQ_Quantum_Ultra;
         case InstrumentModelType_TSQ_Quantum_Ultra_AM:      return MS_TSQ_Quantum_Ultra_AM;
         case InstrumentModelType_TSQ_Vantage_Standard:      return MS_TSQ_Vantage;
+        case InstrumentModelType_TSQ_Vantage_EMR:           return MS_TSQ_Vantage;
         case InstrumentModelType_Element_XR:                return MS_Element_XR;
         case InstrumentModelType_Element_GD:                return MS_Element_GD;
         case InstrumentModelType_GC_IsoLink:                return MS_GC_IsoLink;
@@ -250,6 +251,7 @@ vector<InstrumentConfiguration> createInstrumentConfigurations(const Component&
         case InstrumentModelType_TSQ_Quantum_Ultra:
         case InstrumentModelType_TSQ_Quantum_Ultra_AM:
         case InstrumentModelType_TSQ_Vantage_Standard:
+        case InstrumentModelType_TSQ_Vantage_EMR:
         case InstrumentModelType_GC_Quantum:
             configurations.push_back(InstrumentConfiguration());
             configurations.back().componentList.push_back(commonSource);
