@@ -1,5 +1,5 @@
 //
-// $Id: Reader_Thermo_Test.cpp 6478 2014-07-08 20:01:38Z chambm $
+// $Id: Reader_Thermo_Test.cpp 7155 2015-02-03 22:38:32Z chambm $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -185,7 +185,8 @@ int main(int argc, char* argv[])
         unit_assert(allInstrumentTestsPassed);
         #endif
 
-        pwiz::util::testReader(pwiz::msdata::Reader_Thermo(), testArgs, testAcceptOnly, IsRawFile());
+        bool requireUnicodeSupport = true;
+        pwiz::util::testReader(pwiz::msdata::Reader_Thermo(), testArgs, testAcceptOnly, requireUnicodeSupport, IsRawFile());
     }
     catch (exception& e)
     {
