@@ -1,5 +1,5 @@
 //
-// $Id: SpectrumList_Thermo.cpp 6585 2014-08-07 22:49:28Z chambm $
+// $Id: SpectrumList_Thermo.cpp 9161 2015-11-20 01:04:18Z pcbrefugee $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -160,11 +160,15 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
     {
         rawfile_->setCurrentController(ie.controllerType, ie.controllerNumber);
     }
-    catch (RawEgg&)
+    catch (RawEgg& r)
     {
+        const char *what = r.what();
+        if (what == NULL)
+            what = "cause unknown";
         throw runtime_error("[SpectrumList_Thermo::spectrum()] Error setting controller to: " +
                             lexical_cast<string>(ie.controllerType) + "," +
-                            lexical_cast<string>(ie.controllerNumber));
+                            lexical_cast<string>(ie.controllerNumber) +
+                            " (" + what + ")");
     }
 
     // allocate a new Spectrum
@@ -319,10 +323,7 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
         result->set(MS_total_ion_current, scanInfo->totalIonCurrent());
 
         if (scanInfo->FAIMSOn())
-		{
-            result->set(MS_FAIMS);
             result->set(MS_FAIMS_compensation_voltage, scanInfo->CompensationVoltage());
-        }
 
         size_t scanRangeCount = scanInfo->scanRangeCount();
         if ((scanType == ScanType_SIM || scanType == ScanType_SRM) && scanRangeCount > 1)
@@ -378,7 +379,7 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
             !scanInfo->hasMultiplePrecursors() &&
             precursorCount != msLevel-1)
         {
-            throw runtime_error("[SpectrumList_Thermo::spectrum()] Precursor count does not match ms level.");
+            throw runtime_error("precursor count does not match ms level");
         }
 
         if (scanInfo->hasMultiplePrecursors())
@@ -386,7 +387,7 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
             vector<double> isolationWidths = rawfile_->getIsolationWidths(ie.scan);
             if (precursorCount != (long) isolationWidths.size())
             {
-                throw runtime_error("[SpectrumList_Thermo::spectrum()] Precursor count does not match isolation width count.");
+                throw runtime_error("precursor count does not match isolation width count");
             }
 
             Precursor precursor;
@@ -510,8 +511,11 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
                 }
 
                 // if the monoisotopic m/z is outside the isolation window (due to Thermo firmware bug), reset it to isolation m/z
-                if (isolationWidth == 0 && (selectedIonMz < (isolationMz-defaultIsolationWindowLowerOffset)) || (selectedIonMz > (isolationMz+defaultIsolationWindowUpperOffset)))
-                    selectedIonMz = isolationMz;
+                if (isolationWidth <= 2.0)
+                {
+                    if ((selectedIonMz < (isolationMz - defaultIsolationWindowLowerOffset * 2)) || (selectedIonMz >(isolationMz + defaultIsolationWindowUpperOffset)))
+                        selectedIonMz = isolationMz;
+                }
                 else if ((selectedIonMz < (isolationMz-isolationWidth)) || (selectedIonMz > (isolationMz+isolationWidth)))
                     selectedIonMz = isolationMz;
 
@@ -549,7 +553,7 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
                                                                               precursorScanTime, precursorScanTime,
                                                                               Smoothing_None, 0);
                         if (c->size() == 0)
-                            throw runtime_error("[SpectrumList_Thermo::spectrum()] No chromatogram time points for the scan time.");
+                            throw runtime_error("no chromatogram time points for the scan time");
 
                         if (c->data()[0].intensity > 0)
                             selectedIon.set(MS_peak_intensity, c->data()[0].intensity, MS_number_of_detector_counts);
@@ -619,6 +623,10 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
     {
         throw runtime_error("[SpectrumList_Thermo::spectrum()] Error retrieving spectrum \"" + result->id + "\": " + e.what());
     }
+    catch (exception& e)
+    {
+        throw runtime_error("[SpectrumList_Thermo::spectrum()] Error retrieving spectrum \"" + result->id + "\": " + e.what());
+    }
     catch (...)
     {
         throw runtime_error("[SpectrumList_Thermo::spectrum()] Unknown exception retrieving spectrum \"" + result->id + "\"");
