@@ -1,5 +1,5 @@
 //
-// $Id: Reader_Waters.cpp 6478 2014-07-08 20:01:38Z chambm $
+// $Id: Reader_Waters.cpp 8961 2015-10-08 15:03:23Z chambm $
 //
 //
 // Original author: Matt Chambers <matt.chambers .@. vanderbilt.edu>
@@ -106,6 +106,10 @@ void fillInMetadata(const string& rawpath, RawDataPtr rawdata, MSData& msd)
         if (find(functionFilepaths.begin(), functionFilepaths.end(), itr->path()) != functionFilepaths.end())
             continue;
 
+        // skip lockmass cache
+        if (bal::iends_with(itr->path().string(), "lmgt.inf"))
+            continue;
+
         bfs::path sourcePath = itr->path();
         SourceFilePtr sourceFile(new SourceFile);
         sourceFile->id = BFS_STRING(sourcePath.leaf());
